\name{qa}
\alias{qa}
\alias{qa,character-method}

\title{Perform quality assessment on short reads}

\description{

  This function is a common interface to quality assessment functions
  available in \code{ShortRead}. Results from this function may be
  displayed in brief, or integrated into reports using, e.g.,
  \code{\link{report}}.

}
\usage{
qa(dirPath, ...)
}

\arguments{

  \item{dirPath}{A character vector or other object (e.g.,
    \code{\link{SolexaPath}}; see \code{showMethods}, below) locating
    the data for which quality assessment is to be performed. See help
    pages for defined methods (by evaluating the example code, below)
    for details of available methods.}

  \item{\dots}{Additional arguments used by methods.}
}

\details{
  The following methods are defined:
  \itemize{
    \item{
      \code{dirPath="character", pattern="character",
		type=c("SolexaExport", "Bowtie", "MAQMapShort", "fastq"), ...}}{

      Quality assessment is performed on all files in directory
      \code{dirPath} whose file name matches \code{pattern}. The type of
      analysis performed is based on the \code{type} argument. Use
      \code{SolexaExport} when all files matching \code{pattern} are
      Solexa \code{_export.txt} files. Use \code{Bowtie} for Bowtie
      files. Use \code{MAQMapShort} for MAQ \code{map} files produced by
      MAQ versions below 0.70, and \code{fastq} for collections of
      fastq-format files. Quality assessment details vary depending on
      data source.

    }
  }
}

\value{
  An object derived from class \code{\linkS4class{.QA}}
}

\author{Martin Morgan <mtmorgan@fhcrc.org>}

\seealso{
  \code{\linkS4class{.QA}},
  \code{\linkS4class{SolexaExportQA}}
  \code{\linkS4class{MAQMapQA}}
  \code{\linkS4class{FastqQA}}
}

\examples{
showMethods("qa")
}

\keyword{manip}
